# -*- coding: utf-8 -*-
"""
Miscellaneous utilities and tools
"""
from __future__ import absolute_import, print_function

import functools
import keyword
import os
import re
import sys
from contextlib import contextmanager
from operator import itemgetter
from distutils.filelist import FileList

from six import PY2

from .templates import licenses


@contextmanager
def chdir(path):
    """
    Contextmanager to change into a directory

    :param path: path to change into as string
    """
    curr_dir = os.getcwd()
    os.chdir(path)
    try:
        yield
    finally:
        os.chdir(curr_dir)


def is_valid_identifier(string):
    """
    Check if string is a valid package name

    :param string: package name as string
    :return: boolean
    """
    if not re.match("[_A-Za-z][_a-zA-Z0-9]*$", string):
        return False
    if keyword.iskeyword(string):
        return False
    return True


def make_valid_identifier(string):
    """
    Try to make a valid package name identifier from a string

    :param string: invalid package name as string
    :return: valid package name as string or :obj:`RuntimeError`
    """
    string = string.strip()
    string = string.replace("-", "_")
    string = string.replace(" ", "_")
    string = re.sub('[^_a-zA-Z0-9]', '', string)
    string = string.lower()
    if is_valid_identifier(string):
        return string
    else:
        raise RuntimeError("String cannot be converted to a valid identifier.")


def list2str(lst, indent=0, brackets=True, quotes=True):
    """
    Generate a Python syntax list string with an indention

    :param lst: list
    :param indent: indention as integer
    :param brackets: surround the list expression by brackets as boolean
    :param quotes: surround each item with quotes
    :return: string
    """
    if quotes:
        lst_str = str(lst)
        if not brackets:
            lst_str = lst_str[1:-1]
    else:
        lst_str = ', '.join(lst)
        if brackets:
            lst_str = '[' + lst_str + ']'
    lb = ',\n' + indent*' '
    return lst_str.replace(', ', lb)


def exceptions2exit(exception_list):
    """
    Decorator to convert given exceptions to exit messages

    This avoids displaying nasty stack traces to end-users

    :param exception_list: list of exceptions to convert
    """
    def exceptions2exit_decorator(func):
        @functools.wraps(func)
        def func_wrapper(*args, **kwargs):
            try:
                func(*args, **kwargs)
            except tuple(exception_list) as e:
                print(e)
                sys.exit(1)
        return func_wrapper
    return exceptions2exit_decorator


# from http://en.wikibooks.org/, Creative Commons Attribution-ShareAlike 3.0
def levenshtein(s1, s2):
    """
    Calculate the Levenshtein distance between two strings

    :param s1: first string
    :param s2: second string
    :return: distance between s1 and s2 as integer
    """
    if len(s1) < len(s2):
        return levenshtein(s2, s1)

    # len(s1) >= len(s2)
    if len(s2) == 0:
        return len(s1)

    previous_row = range(len(s2) + 1)
    for i, c1 in enumerate(s1):
        current_row = [i + 1]
        for j, c2 in enumerate(s2):
            insertions = previous_row[j + 1] + 1
            deletions = current_row[j] + 1
            substitutions = previous_row[j] + (c1 != c2)
            current_row.append(min(insertions, deletions, substitutions))
        previous_row = current_row

    return previous_row[-1]


def best_fit_license(txt):
    """
    Finds proper license name for the license defined in txt

    :param txt: license name as string
    :return: license name as string
    """
    ratings = {lic: levenshtein(txt, lic.lower()) for lic in licenses}
    return min(ratings.items(), key=itemgetter(1))[0]


def utf8_encode(string):
    """
    Encode a Python 2 unicode object to str for compatibility with Python 3

    :param string: Python 2 unicode object or Python 3 str object
    :return: Python 2 str object or Python 3 str object
    """
    return string.encode(encoding='utf8') if PY2 else string


def utf8_decode(string):
    """
    Decode a Python 2 str object to unicode for compatibility with Python 3

    :param string: Python 2 str object or Python 3 str object
    :return: Python 2 unicode object or Python 3 str object
    """
    return string.decode(encoding='utf8') if PY2 else string


def get_files(pattern):
    """
    Retrieve all files in the current directory by a pattern.
    Use ** as greedy wildcard and * as non-greedy wildcard.

    :param pattern: The pattern as used by :obj:`distutils.filelist.Filelist`
    """
    filelist = FileList()
    if '**' in pattern:
        pattern = pattern.replace('**', '*')
        anchor = False
    else:
        anchor = True
    filelist.include_pattern(pattern, anchor)
    return filelist.files


def prepare_namespace(namespace_str):
    """
    Check the validity of namespace_str and split it up into a list

    :param namespace_str: namespace as string, e.g. "com.blue_yonder"
    :return: list of namespaces, e.g. ["com", "com.blue_yonder"]
    """
    namespaces = namespace_str.split('.') if namespace_str else list()
    for namespace in namespaces:
        if not is_valid_identifier(namespace):
            raise RuntimeError(
                "{} is not a valid namespace package.".format(namespace))
    return ['.'.join(namespaces[:i+1]) for i in range(len(namespaces))]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import stat
import tempfile
from imp import reload
from shutil import rmtree
from subprocess import CalledProcessError

import pytest
import pkg_resources
import pyscaffold
from pyscaffold import shell

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def set_writable(func, path, exc_info):
    if not os.access(path, os.W_OK):
        os.chmod(path, stat.S_IWUSR)
        func(path)
    else:
        raise


@pytest.yield_fixture()
def tmpdir():
    old_path = os.getcwd()
    newpath = tempfile.mkdtemp()
    os.chdir(newpath)
    try:
        yield
    finally:
        os.chdir(old_path)
        rmtree(newpath, onerror=set_writable)


@pytest.yield_fixture()
def git_mock():
    def mock(*_):
        yield "git@mock"

    old_git = shell.git
    shell.git = mock
    try:
        yield
    finally:
        shell.git = old_git


@pytest.yield_fixture()
def nogit_mock():
    def raise_error(*_):
        raise CalledProcessError(1, "git", "No git mock!")

    old_git = shell.git
    shell.git = raise_error
    try:
        yield
    finally:
        shell.git = old_git


@pytest.yield_fixture()
def noconfgit_mock():
    def raise_error(*argv):
        if 'config' in argv:
            raise CalledProcessError(1, "git", "No git mock!")

    old_git = shell.git
    shell.git = raise_error
    try:
        yield
    finally:
        shell.git = old_git


@pytest.yield_fixture()
def nodjango_admin_mock():
    def raise_error(*_):
        raise CalledProcessError(1, "django_admin.py", "No django_admin mock!")

    old_django_admin = shell.django_admin
    shell.django_admin = raise_error
    try:
        yield
    finally:
        shell.django_admin = old_django_admin


@pytest.yield_fixture()
def nosphinx_mock():
    try:
        import builtins
    except ImportError:
        import __builtin__ as builtins
    realimport = builtins.__import__

    def my_import(name, *args):
        if name.startswith('sphinx'):
            raise ImportError
        return realimport(name, *args)

    try:
        builtins.__import__ = my_import
        yield
    finally:
        builtins.__import__ = realimport


@pytest.yield_fixture()
def get_distribution_raises_exception():
    def raise_exeception():
        raise RuntimeError("No get_distribution mock")
    orig_get_distribution = pkg_resources.get_distribution
    pkg_resources.get_distribution = raise_exeception
    reload(pyscaffold)
    try:
        yield
    finally:
        pkg_resources.get_distribution = orig_get_distribution
        reload(pyscaffold)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from subprocess import CalledProcessError

import pytest
from pyscaffold import shell

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def test_ShellCommand():
    echo = shell.ShellCommand('echo')
    output = echo('Hello Echo!!!')
    assert next(output).strip('"') == 'Hello Echo!!!'
    python = shell.ShellCommand('python')
    output = python('-c', 'print("Hello World")')
    assert list(output)[-1] == 'Hello World'


def test_called_process_error2exit_decorator():
    @shell.called_process_error2exit_decorator
    def func(_):
        raise CalledProcessError(1, "command", "wrong input!")
    with pytest.raises(SystemExit):
        func(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from os.path import isdir, isfile

import pytest
import six
from pyscaffold import cli, structure, utils

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def test_create_structure(tmpdir):  # noqa
    struct = {"my_file": "Some content",
              "my_folder": {
                  "my_dir_file": "Some other content"
              },
              "empty_folder": {}}
    structure.create_structure(struct)

    assert isdir("my_folder")
    assert isdir("empty_folder")
    assert isfile("my_folder/my_dir_file")
    assert isfile("my_file")
    assert open("my_file").read() == "Some content"
    assert open("my_folder/my_dir_file").read() == "Some other content"


def test_create_structure_with_wrong_type(tmpdir):  # noqa
    with pytest.raises(RuntimeError):
        struct = {"strange_thing": 1}
        structure.create_structure(struct)


def test_create_structure_when_updating(tmpdir):  # noqa
    struct = {"my_file": "Some content",
              "my_folder": {
                  "my_dir_file": "Some other content"
              },
              "empty_folder": {}}
    structure.create_structure(struct, update=False)
    struct["my_folder"]["my_dir_file"] = "Changed content"
    structure.create_structure(struct, update=True)
    with open("my_folder/my_dir_file") as fh:
        assert fh.read() == "Changed content"


def test_create_structure_when_dir_exists(tmpdir):  # noqa
    struct = {"my_folder": {"my_dir_file": "Some other content"}}
    os.mkdir("my_folder")
    with pytest.raises(OSError):
        structure.create_structure(struct, update=False)


def test_make_structure():
    args = ["project", "-p", "package", "-d", "description"]
    opts = cli.parse_args(args)
    opts = cli.get_default_opts(opts['project'], **opts)
    struct = structure.make_structure(opts)
    assert isinstance(struct, dict)


def test_create_django_project_no_django(nodjango_admin_mock):  # noqa
    args = ["project", "-p", "package", "-d", "description"]
    opts = cli.parse_args(args)
    with pytest.raises(RuntimeError):
        structure.create_django_proj(opts)


def test_make_structure_with_pre_commit_hooks():
    args = ["project",
            "-p", "package",
            "-d", "description",
            "--with-pre-commit"]
    opts = cli.parse_args(args)
    opts = cli.get_default_opts(opts['project'], **opts)
    struct = structure.make_structure(opts)
    assert isinstance(struct, dict)
    assert ".pre-commit-config.yaml" in struct["project"]


def test_make_structure_with_tox():
    args = ["project",
            "-p", "package",
            "-d", "description",
            "--with-tox"]
    opts = cli.parse_args(args)
    opts = cli.get_default_opts(opts['project'], **opts)
    struct = structure.make_structure(opts)
    assert isinstance(struct, dict)
    assert "tox.ini" in struct["project"]
    assert isinstance(struct["project"]["tox.ini"], six.string_types)


def test_apply_update_rules(tmpdir):  # noqa
    NO_OVERWRITE = structure.FileOp.NO_OVERWRITE
    NO_CREATE = structure.FileOp.NO_CREATE
    rules = {"a": NO_OVERWRITE,
             "c": {"b": NO_OVERWRITE},
             "d": {"a": NO_OVERWRITE,
                   "b": NO_CREATE,
                   "c": NO_CREATE,
                   "d": NO_OVERWRITE},
             "e": NO_CREATE}
    struct = {"a": "a",
              "b": "b",
              "c": {"a": "a",
                    "b": "b"},
              "d": {"a": "a",
                    "b": "b"},
              "e": "e"}
    dir_struct = {"a": "a",
                  "c": {"b": "b"}}
    exp_struct = {"b": "b",
                  "c": {"a": "a"},
                  "d": {"a": "a"}}
    structure.create_structure(dir_struct)
    res_struct = structure.apply_update_rules(rules, struct)
    assert res_struct == exp_struct


def test_add_namespace():
    args = ["project",
            "-p", "package",
            "--with-namespace", "com.blue_yonder"]
    opts = cli.parse_args(args)
    opts['namespace'] = utils.prepare_namespace(opts['namespace'])
    struct = {"project": {"package": {"file1": "Content"}}}
    ns_struct = structure.add_namespace(opts, struct)
    assert ["project"] == list(ns_struct.keys())
    assert "package" not in list(ns_struct.keys())
    assert ["com"] == list(ns_struct["project"].keys())
    assert {"blue_yonder", "__init__.py"} == set(
        ns_struct["project"]["com"].keys())
    assert "package" in list(ns_struct["project"]["com"]["blue_yonder"].keys())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pkg_resources
import pyscaffold

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def test_version():
    version = pyscaffold.__version__.split(".")
    assert int(version[0]) >= 0


def test_unknown_version(get_distribution_raises_exception):  # noqa
    version = pyscaffold.__version__
    assert version == 'unknown'



# -*- coding: utf-8 -*-
"""
Functionality to generate and work with the directory structure of a project
"""
from __future__ import absolute_import, print_function

import os
from os.path import join as join_path

from six import string_types

from . import shell, templates, utils

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


class FileOp(object):
    """
    Namespace for file operations during an update

    NO_OVERWRITE: Do not overwrite an existing file during update
    NO_CREATE: Do not create the file during an update
    """
    NO_OVERWRITE = 0
    NO_CREATE = 1


def add_namespace(opts, struct):
    """
    Prepend the namespace to a given file structure

    :param opts: options as dictionary
    :param struct: directory structure as dictionary of dictionaries
    :return: directory structure as dictionary of dictionaries
    """
    if not opts['namespace']:
        return struct
    namespace = opts['namespace'][-1].split('.')
    base_struct = struct
    pkg_struct = struct[opts['project']][opts['package']]
    struct = base_struct[opts['project']]
    del struct[opts['package']]
    for sub_package in namespace:
        struct[sub_package] = {'__init__.py': templates.namespace(opts)}
        struct = struct[sub_package]
    struct[opts['package']] = pkg_struct
    return base_struct


def make_structure(opts):
    """
    Creates the project structure as dictionary of dictionaries

    :param opts: options as dictionary
    :return: structure as dictionary of dictionaries
    """
    struct = {opts['project']: {
        '.gitignore': templates.gitignore(opts),
        opts['package']: {'__init__.py': templates.init(opts),
                          'skeleton.py': templates.skeleton(opts)},
        'tests': {'conftest.py': templates.conftest_py(opts)},
        'docs': {'conf.py': templates.sphinx_conf(opts),
                 'authors.rst': templates.sphinx_authors(opts),
                 'index.rst': templates.sphinx_index(opts),
                 'license.rst': templates.sphinx_license(opts),
                 'changes.rst': templates.sphinx_changes(opts),
                 'Makefile': templates.sphinx_makefile(opts),
                 '_static': {
                     '.gitignore': templates.gitignore_empty(opts)}},
        'README.rst': templates.readme(opts),
        'AUTHORS.rst': templates.authors(opts),
        'LICENSE.txt': templates.license(opts),
        'CHANGES.rst': templates.changes(opts),
        'setup.py': templates.setup_py(opts),
        'setup.cfg': templates.setup_cfg(opts),
        'requirements.txt': templates.requirements(opts),
        '.coveragerc': templates.coveragerc(opts)}}
    proj_dir = struct[opts['project']]
    if opts['travis']:
        proj_dir['.travis.yml'] = templates.travis(opts)
        proj_dir['tests']['travis_install.sh'] = templates.travis_install(opts)
    if opts['pre_commit']:
        proj_dir['.pre-commit-config.yaml'] = templates.pre_commit_config(opts)
    if opts['tox']:
        proj_dir['tox.ini'] = templates.tox(opts)
    if opts['update'] and not opts['force']:
        # Do not overwrite following files
        rules = {opts['project']: {
            '.gitignore': FileOp.NO_OVERWRITE,
            '.gitattributes': FileOp.NO_OVERWRITE,
            'setup.cfg': FileOp.NO_OVERWRITE,
            'README.rst': FileOp.NO_OVERWRITE,
            'CHANGES.rst': FileOp.NO_OVERWRITE,
            'LICENSE.txt': FileOp.NO_OVERWRITE,
            'AUTHORS.rst': FileOp.NO_OVERWRITE,
            'requirements.txt': FileOp.NO_OVERWRITE,
            '.travis.yml': FileOp.NO_OVERWRITE,
            '.coveragerc': FileOp.NO_OVERWRITE,
            '.pre-commit-config.yaml': FileOp.NO_OVERWRITE,
            'tox.ini': FileOp.NO_OVERWRITE,
            opts['package']: {'skeleton.py': FileOp.NO_CREATE},
            'tests': {'conftest.py': FileOp.NO_OVERWRITE,
                      'travis_install.sh': FileOp.NO_OVERWRITE},
            'docs': {'index.rst': FileOp.NO_OVERWRITE}
        }}
        struct = apply_update_rules(rules, struct)
    struct = add_namespace(opts, struct)

    return struct


def create_structure(struct, prefix=None, update=False):
    """
    Manifests a directory structure in the filesystem

    :param struct: directory structure as dictionary of dictionaries
    :param prefix: prefix path for the structure
    :param update: update an existing directory structure as boolean
    """
    if prefix is None:
        prefix = os.getcwd()
    for name, content in struct.items():
        if isinstance(content, string_types):
            with open(join_path(prefix, name), 'w') as fh:
                fh.write(utils.utf8_encode(content))
        elif isinstance(content, dict):
            try:
                os.mkdir(join_path(prefix, name))
            except OSError:
                if not update:
                    raise
            create_structure(struct[name],
                             prefix=join_path(prefix, name),
                             update=update)
        elif content is None:
            pass
        else:
            raise RuntimeError("Don't know what to do with content type "
                               "{type}.".format(type=type(content)))


def create_django_proj(opts):
    """
    Creates a standard Django project with django-admin.py

    :param opts: options as dictionary
    """
    try:
        shell.django_admin('--version')
    except:
        raise RuntimeError("django-admin.py is not installed, "
                           "run: pip install django")
    shell.django_admin('startproject', opts['project'])


def create_cookiecutter(opts):
    """
    Create a cookie cutter template

    :param opts: options as dictionary
    """
    try:
        from cookiecutter.main import cookiecutter
    except:
        raise RuntimeError("cookiecutter is not installed, "
                           "run: pip install cookiecutter")
    extra_context = dict(full_name=opts['author'],
                         email=opts['email'],
                         project_name=opts['project'],
                         repo_name=opts['package'],
                         project_short_description=opts['description'],
                         release_date=opts['release_date'],
                         version='unknown',  # will be replaced later
                         year=opts['year'])

    cookiecutter(opts['cookiecutter_template'],
                 no_input=True,
                 extra_context=extra_context)


def apply_update_rules(rules, struct, prefix=None):
    """
    Apply update rules using :obj:`~.FileOp` to a directory structure

    :param rules: directory structure as dictionary of dictionaries with
        :obj:`~.FileOp` keys. The structure will be modified.
    :param struct: directory structure as dictionary of dictionaries
    :param prefix: prefix path for the structure
    :return: directory structure with keys removed according to the rules
    """
    if prefix is None:
        prefix = os.getcwd()
    for k, v in rules.items():
        if isinstance(v, dict):
            apply_update_rules(v, struct[k], join_path(prefix, k))
        else:
            path = join_path(prefix, k)
            if v == FileOp.NO_OVERWRITE and os.path.exists(path):
                struct.pop(k, None)
            elif v == FileOp.NO_CREATE:
                struct.pop(k, None)
    return struct



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import tempfile

import pytest
import six
from pyscaffold import utils, templates
from pyscaffold.structure import create_structure


def test_chdir():
    curr_dir = os.getcwd()
    try:
        temp_dir = tempfile.mkdtemp()
        with utils.chdir(temp_dir):
            new_dir = os.getcwd()
        assert new_dir == os.path.realpath(temp_dir)
        assert curr_dir == os.getcwd()
    finally:
        os.rmdir(temp_dir)


def test_is_valid_identifier():
    bad_names = ["has whitespace",
                 "has-hyphen",
                 "has_special_char$",
                 "1starts_with_digit"]
    for bad_name in bad_names:
        assert not utils.is_valid_identifier(bad_name)
    valid_names = ["normal_variable_name",
                   "_private_var",
                   "_with_number1"]
    for valid_name in valid_names:
        assert utils.is_valid_identifier(valid_name)


def test_make_valid_identifier():
    assert utils.make_valid_identifier("has whitespaces ") == "has_whitespaces"
    assert utils.make_valid_identifier("has-hyphon") == "has_hyphon"
    assert utils.make_valid_identifier("special chars%") == "special_chars"
    assert utils.make_valid_identifier("UpperCase") == "uppercase"
    with pytest.raises(RuntimeError):
        utils.make_valid_identifier("def")


def test_list2str():
    classifiers = ['Development Status :: 4 - Beta',
                   'Programming Language :: Python']
    class_str = utils.list2str(classifiers, indent=len("classifiers = ") + 1)
    exp_class_str = """\
['Development Status :: 4 - Beta',
               'Programming Language :: Python']"""
    assert class_str == exp_class_str
    classifiers = ['Development Status :: 4 - Beta']
    class_str = utils.list2str(classifiers, indent=len("classifiers = ") + 1)
    assert class_str == "['Development Status :: 4 - Beta']"
    classifiers = []
    class_str = utils.list2str(classifiers, indent=len("classifiers = ") + 1)
    assert class_str == "[]"
    classifiers = ['Development Status :: 4 - Beta']
    class_str = utils.list2str(classifiers, brackets=False)
    assert class_str == "'Development Status :: 4 - Beta'"
    class_str = utils.list2str(classifiers, brackets=False, quotes=False)
    assert class_str == "Development Status :: 4 - Beta"
    class_str = utils.list2str(classifiers, brackets=True, quotes=False)
    assert class_str == "[Development Status :: 4 - Beta]"


def test_exceptions2exit():
    @utils.exceptions2exit([RuntimeError])
    def func(_):
        raise RuntimeError("Exception raised")
    with pytest.raises(SystemExit):
        func(1)


def test_levenshtein():
    s1 = "born"
    s2 = "burn"
    assert utils.levenshtein(s1, s2) == 1
    s2 = "burnt"
    assert utils.levenshtein(s1, s2) == 2
    assert utils.levenshtein(s2, s1) == 2
    s2 = ""
    assert utils.levenshtein(s2, s1) == 4


def test_utf8_encode():
    s_in = six.u('äüä')
    s_out = utils.utf8_encode(s_in)
    assert isinstance(s_out, six.string_types)


def test_utf8_decode():
    s_in = "äüä"
    s_out = utils.utf8_decode(s_in)
    assert isinstance(s_out, six.string_types)


def test_get_files(tmpdir):  # noqa
    struct = {'subdir': {'script.py': '#Python script...'},
              'root_script.py': '#Root Python script...'}
    create_structure(struct)
    files = utils.get_files("*.py")
    assert 'root_script.py' in files
    assert 'subdir/script.py' not in files
    files = utils.get_files("**.py")
    assert 'root_script.py' in files
    assert 'subdir/script.py' in files


def test_prepare_namespace():
    namespaces = utils.prepare_namespace("com")
    assert namespaces == ["com"]
    namespaces = utils.prepare_namespace("com.blue_yonder")
    assert namespaces == ["com", "com.blue_yonder"]
    with pytest.raises(RuntimeError):
        utils.prepare_namespace("com.blue-yonder")


def test_best_fit_license():
    txt = "new_bsd"
    assert utils.best_fit_license(txt) == "new-bsd"
    for license in templates.licenses.keys():
        assert utils.best_fit_license(license) == license



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from pyscaffold import templates

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def test_get_template():
    template = templates.get_template("setup_py")
    content = template.safe_substitute()
    assert content.split(os.linesep, 1)[0] == '#!/usr/bin/env python'


def test_all_licenses():
    opts = {"email": "test@user",
            "project": "my_project",
            "author": "myself",
            "year": 1832}
    for license in templates.licenses.keys():
        opts['license'] = license
        assert templates.license(opts)



# -*- coding: utf-8 -*-

"""Runner module for demoapp"""

from __future__ import absolute_import, division, print_function

import argparse
import sys

import demoapp

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def parse_args(args):
    """
    Parse command line parameters

    :param args: command line parameters as list of strings
    :return: command line parameters as :obj:`argparse.Namespace`
    """
    parser = argparse.ArgumentParser(
        description="A demo application for PyScaffold's unit testing")
    version = demoapp.__version__
    parser.add_argument('-v',
                        '--version',
                        action='version',
                        version='demoapp {ver}'.format(ver=version))
    opts = parser.parse_args(args)
    return opts


def main(args):
    parse_args(args)
    print("Hello World")


def run():
    """
    Entry point for setup.py
    """
    main(sys.argv[1:])


if __name__ == '__main__':
    main(sys.argv[1:])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os.path

import pytest
from pyscaffold import repo, structure

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def test_init_commit_repo(tmpdir):  # noqa
    project = "my_project"
    struct = {project: {
        "my_file": "Some other content",
        "my_dir": {"my_file": "Some more content"},
        "dummy": None}
    }
    structure.create_structure(struct)
    dummy_file = os.path.join(project, "dummy")
    with open(dummy_file, 'w'):
        os.utime(dummy_file, None)
    repo.init_commit_repo(project, struct)
    assert os.path.exists(os.path.join(project, ".git"))


def test_init_commit_repo_with_wrong_structure(tmpdir):  # noqa
    project = "my_project"
    struct = {project: {
        "my_file": type("StrangeType", (object,), {})}}
    os.mkdir(project)
    with pytest.raises(RuntimeError):
        repo.init_commit_repo(project, struct)


def test_add_tag(tmpdir):  # noqa
    project = "my_project"
    struct = {project: {
        "my_file": "Some other content",
        "my_dir": {"my_file": "Some more content"}}
    }
    structure.create_structure(struct)
    repo.init_commit_repo(project, struct)
    repo.add_tag(project, "v0.0")
    repo.add_tag(project, "v0.1", "Message with whitespace")



# -*- coding: utf-8 -*-

"""Unittests for the integration part of PyScaffold"""

from __future__ import division, print_function, absolute_import

import os
from distutils.dist import Distribution
from distutils.cmd import Command

from pyscaffold import integration
from setuptools_scm.version import ScmVersion


__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def test_version2str():
    ver = ScmVersion('1.0', distance=0)
    ver_str = integration.version2str(ver)
    assert ver_str == '1.0'
    ver = ScmVersion('1.0', distance=1)
    ver_str = integration.version2str(ver)
    assert ver_str == '1.0.post0.dev1'
    ver = ScmVersion('1.0.dev0', distance=1)
    ver_str = integration.version2str(ver)
    assert ver_str == '1.0.post0.dev1'


def test_local_version2str():
    ver = ScmVersion('1.0', dirty=True, node='abcdef')
    ver_str = integration.local_version2str(ver)
    assert ver_str == '+abcdef.dirty'
    ver = ScmVersion('1.0', dirty=False)
    ver_str = integration.local_version2str(ver)
    assert ver_str == ''
    ver = ScmVersion('1.0', distance=1, dirty=True, node='abcdef')
    ver_str = integration.local_version2str(ver)
    assert ver_str == '+abcdef.dirty'
    ver = ScmVersion('1.0', distance=1, dirty=False, node='abcdef')
    ver_str = integration.local_version2str(ver)
    assert ver_str == '+abcdef'


def test_build_cmd_docs():
    cmd = integration.build_cmd_docs()
    assert issubclass(cmd, Command)


def test_build_cmd_docs_no_sphinx(nosphinx_mock):  # noqa
    cmd = integration.build_cmd_docs()
    assert cmd.__name__ == 'NoSphinx'


def test_deactivate_pbr_authors_changelog():
    integration.deactivate_pbr_authors_changelog()
    assert os.environ['SKIP_GENERATE_AUTHORS'] == "1"
    assert os.environ['SKIP_WRITE_GIT_CHANGELOG'] == "1"


def test_pyscaffold_keyword():  # noqa
    dist = Distribution()
    integration.pyscaffold_keyword(dist, 'use_pyscaffold', True)



# -*- coding: utf-8 -*-

"""Runner module for demoapp_data"""

from __future__ import absolute_import, division, print_function

import argparse
import os
import sys
from pkgutil import get_data

import demoapp_data

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def get_hello_world():
    pkg_name = __name__.split('.', 1)[0]
    data = get_data(pkg_name, os.path.join('data', 'hello_world.txt'))
    if sys.version_info[0] >= 3:
        data = data.decode()
    return data


def parse_args(args):
    """
    Parse command line parameters

    :param args: command line parameters as list of strings
    :return: command line parameters as :obj:`argparse.Namespace`
    """
    parser = argparse.ArgumentParser(
        description="A demo application for PyScaffold's unit testing")
    version = demoapp_data.__version__
    parser.add_argument('-v',
                        '--version',
                        action='version',
                        version='demoapp_data {ver}'.format(ver=version))
    opts = parser.parse_args(args)
    return opts


def main(args):
    parse_args(args)
    hello_world = get_hello_world()
    print(hello_world)


def run():
    """
    Entry point for setup.py
    """
    main(sys.argv[1:])


if __name__ == '__main__':
    main(sys.argv[1:])



# -*- coding: utf-8 -*-
"""
Templates for all files of a project's scaffold
"""
from __future__ import absolute_import, print_function

import os.path
import string
from pkgutil import get_data

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"

licenses = {"affero": "license_affero_3.0",
            "apache": "license_apache",
            "artistic": "license_artistic_2.0",
            "cc0": "license_cc0_1.0",
            "eclipse": "license_eclipse_1.0",
            "gpl2": "license_gpl_2.0",
            "gpl3": "license_gpl_3.0",
            "isc": "license_isc",
            "lgpl2": "license_lgpl_2.1",
            "lgpl3": "license_lgpl_3.0",
            "mit": "license_mit",
            "mozilla": "license_mozilla",
            "new-bsd": "license_new_bsd",
            "none": "license_none",
            "proprietary": "license_none",
            "public-domain": "license_public_domain",
            "simple-bsd": "license_simplified_bsd"}


def get_template(name):
    """
    Retrieve the template by name

    :param name: name of template
    :return: template as :obj:`string.Template`
    """
    pkg_name = __name__.split(".", 1)[0]
    file_name = "{name}.template".format(name=name)
    data = get_data(pkg_name, os.path.join("data", file_name))
    return string.Template(data.decode(encoding='utf8'))


def setup_py(opts):
    """
    Template of setup.py

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("setup_py")
    return template.safe_substitute(opts)


def setup_cfg(opts):
    """
    Template of setup.cfg

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("setup_cfg")
    return template.substitute(opts)


def gitignore(opts):
    """
    Template of .gitignore

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("gitignore")
    return template.substitute(opts)


def gitignore_empty(opts):
    """
    Template of empty .gitignore

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("gitignore_empty")
    return template.substitute(opts)


def sphinx_conf(opts):
    """
    Template of conf.py

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("sphinx_conf")
    numpydoc_sphinx_ext = ", 'numpydoc'" if opts['numpydoc'] else ""
    return template.substitute(numpydoc_sphinx_ext=numpydoc_sphinx_ext, **opts)


def sphinx_index(opts):
    """
    Template of index.rst

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("sphinx_index")
    return template.substitute(opts)


def sphinx_license(opts):
    """
    Template of license.rst

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("sphinx_license")
    return template.substitute(opts)


def sphinx_authors(opts):
    """
    Template of authors.rst

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("sphinx_authors")
    return template.substitute(opts)


def sphinx_changes(opts):
    """
    Template of changes.rst

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("sphinx_changes")
    return template.substitute(opts)


def sphinx_makefile(opts):
    """
    Template of Sphinx's Makefile

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("sphinx_makefile")
    return template.safe_substitute(opts)


def readme(opts):
    """
    Template of README.rst

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("readme")
    return template.substitute(opts)


def authors(opts):
    """
    Template of AUTHORS.rst

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("authors")
    return template.substitute(opts)


def requirements(opts):
    """
    Template of requirements.txt

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("requirements")
    return template.substitute(
        requirements_str=',\n'.join(opts['requirements']), **opts)


def license(opts):
    """
    Template of LICENSE.txt

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template(licenses[opts['license']])
    return template.substitute(opts)


def init(opts):
    """
    Template of __init__.py

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("__init__")
    return template.substitute(opts)


def coveragerc(opts):
    """
    Template of .coveragerc

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("coveragerc")
    return template.substitute(opts)


def tox(opts):
    """
    Template of tox.ini

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("tox_ini")
    return template.substitute(opts)


def travis(opts):
    """
    Template of .travis.yml

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("travis")
    return template.safe_substitute(opts)


def travis_install(opts):
    """
    Template of travis_install.sh

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("travis_install")
    return template.safe_substitute(opts)


def pre_commit_config(opts):
    """
    Template of .pre-commit-config.yaml

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("pre-commit-config")
    return template.safe_substitute(opts)


def namespace(opts):
    """
    Template of __init__.py defining a namespace package

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("namespace")
    return template.substitute(opts)


def skeleton(opts):
    """
    Template of skeleton.py defining a basic console script

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("skeleton")
    return template.substitute(opts)


def changes(opts):
    """
    Template of CHANGES.rst

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("changes")
    return template.substitute(opts)


def conftest_py(opts):
    """
    Template of conftest.py

    :param opts: mapping parameters as dictionary
    :return: file content as string
    """
    template = get_template("conftest_py")
    return template.substitute(opts)



# -*- coding: utf-8 -*-

"""Unit tests of everything related to retrieving the version

There are four tree states we want to check:
 A: sitting on the 1.0 tag
 B: dirtying the tree after 1.0
 C: a commit after a tag, clean tree
 D: a commit after a tag, dirty tree
"""

from __future__ import absolute_import, division, print_function

import inspect
import os
import re
import shutil
import sys
from contextlib import contextmanager
from shutil import copyfile, rmtree

import pytest
from pyscaffold import shell
from pyscaffold.repo import add_tag
from pyscaffold.cli import main as putup
from pyscaffold.shell import git
from pyscaffold.utils import chdir

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"

__location__ = os.path.join(os.getcwd(), os.path.dirname(
    inspect.getfile(inspect.currentframe())))


pip = shell.ShellCommand("pip")
setup_py = shell.ShellCommand("python setup.py")
demoapp = shell.ShellCommand("demoapp")
demoapp_data = shell.ShellCommand("demoapp_data")
untar = shell.ShellCommand("tar xvfzk")


def is_inside_venv():
    return hasattr(sys, 'real_prefix')


def check_clean_venv():
    installed = [line.split()[0] for line in pip('list')]
    dirty = ['demoapp', 'demoapp_data', 'UNKNOWN']
    app_list = [x for x in dirty if x in installed]
    if not app_list:
        return
    else:
        raise RuntimeError("Dirty virtual environment:\n{} found".format(
            ', '.join(app_list)))


def create_demoapp(data=False):
    if data:
        demoapp = 'demoapp_data'
    else:
        demoapp = 'demoapp'

    putup([demoapp])
    with chdir(demoapp):
        demoapp_src_dir = os.path.join(__location__, demoapp)
        demoapp_dst_dir = os.path.join(os.getcwd(), demoapp)
        copyfile(os.path.join(demoapp_src_dir, 'runner.py'),
                 os.path.join(demoapp_dst_dir, 'runner.py'))
        git('add', os.path.join(demoapp_dst_dir, 'runner.py'))
        demoapp_dst_dir = os.getcwd()
        copyfile(os.path.join(demoapp_src_dir, 'setup.cfg'),
                 os.path.join(demoapp_dst_dir, 'setup.cfg'))
        git('add', os.path.join(demoapp_dst_dir, 'setup.cfg'))
        if data:
            data_src_dir = os.path.join(demoapp_src_dir, 'data')
            data_dst_dir = os.path.join(os.getcwd(), demoapp, 'data')
            os.mkdir(data_dst_dir)
            copyfile(os.path.join(data_src_dir, 'hello_world.txt'),
                     os.path.join(data_dst_dir, 'hello_world.txt'))
            git('add', os.path.join(data_dst_dir, 'hello_world.txt'))
        git('commit', '-m', 'Added basic progamme logic')


def build_demoapp(dist, path=None, demoapp='demoapp'):
    if path is None:
        path = os.getcwd()
    path = os.path.join(path, demoapp)
    with chdir(path):
        setup_py(dist)


@contextmanager
def installed_demoapp(dist=None, path=None, demoapp='demoapp'):
    check_clean_venv()
    if path is None:
        path = os.getcwd()
    path = os.path.join(path, demoapp, "dist", "{}*".format(demoapp))
    if dist == 'bdist':
        with chdir('/'):
            output = untar(path)
        install_dirs = list()
        install_bin = None
        for line in output:
            if re.search(r".*/site-packages/{}.*?/$".format(demoapp), line):
                install_dirs.append(line)
            if re.search(r".*/bin/{}$".format(demoapp), line):
                install_bin = line
    elif dist == 'install':
        with chdir(demoapp):
            setup_py('install')
    else:
        pip("install", path)
    try:
        yield
    finally:
        if dist == 'bdist':
            with chdir('/'):
                os.remove(install_bin)
                for path in install_dirs:
                    rmtree(path, ignore_errors=True)
        else:
            pip("uninstall", "-y", demoapp)


def check_version(output, exp_version, dirty=False):
    version = output.split(' ')[-1]
    # for some setuptools version a directory with + is generated, sometimes _
    if dirty:
        if '+' in version:
            ver, local = version.split('+')
        else:
            ver, local = version.split('_')
        assert local.endswith('dirty')
        assert ver == exp_version
    else:
        if '+' in version:
            ver = version.split('+')
        else:
            ver = version.split('_')
        if len(ver) > 1:
            assert not ver[1].endswith('dirty')
        assert ver[0] == exp_version


def make_dirty_tree(demoapp='demoapp'):
    dirty_file = os.path.join(demoapp, 'runner.py')
    with chdir(demoapp):
        with open(dirty_file, 'a') as fh:
            fh.write("\n\ndirty_variable = 69\n")


def make_commit(demoapp='demoapp'):
    with chdir(demoapp):
        git('commit', '-a', '-m', 'message')


def rm_git_tree(demoapp='demoapp'):
    git_path = os.path.join(demoapp, '.git')
    shutil.rmtree(git_path)


def test_sdist_install(tmpdir):  # noqa
    create_demoapp()
    build_demoapp('sdist')
    with installed_demoapp():
        out = next(demoapp('--version'))
        exp = "0.0.post0.dev2"
        check_version(out, exp, dirty=False)


def test_sdist_install_dirty(tmpdir):  # noqa
    create_demoapp()
    add_tag('demoapp', 'v0.1', 'first tag')
    make_dirty_tree()
    make_commit()
    make_dirty_tree()
    build_demoapp('sdist')
    with installed_demoapp():
        out = next(demoapp('--version'))
        exp = "0.1.post0.dev1"
        check_version(out, exp, dirty=True)


def test_sdist_install_with_1_0_tag(tmpdir):  # noqa
    create_demoapp()
    make_dirty_tree()
    make_commit()
    add_tag('demoapp', 'v1.0', 'final release')
    build_demoapp('sdist')
    with installed_demoapp():
        out = next(demoapp('--version'))
        exp = "1.0"
        check_version(out, exp, dirty=False)


def test_sdist_install_with_1_0_tag_dirty(tmpdir):  # noqa
    create_demoapp()
    add_tag('demoapp', 'v1.0', 'final release')
    make_dirty_tree()
    build_demoapp('sdist')
    with installed_demoapp():
        out = next(demoapp('--version'))
        exp = "1.0"
        check_version(out, exp, dirty=True)


# bdist works like sdist so we only try one combination
def test_bdist_install(tmpdir):  # noqa
    create_demoapp()
    build_demoapp('bdist')
    with installed_demoapp('bdist'):
        out = next(demoapp('--version'))
        exp = "0.0.post0.dev2"
        check_version(out, exp, dirty=False)


# bdist wheel works like sdist so we only try one combination
@pytest.mark.skipif(not is_inside_venv(),  # noqa
                    reason='Needs to run in a virtualenv')
def test_bdist_wheel_install(tmpdir):
    create_demoapp()
    build_demoapp('bdist_wheel')
    with installed_demoapp():
        out = next(demoapp('--version'))
        exp = "0.0.post0.dev2"
        check_version(out, exp, dirty=False)


def test_git_repo(tmpdir):  # noqa
    create_demoapp()
    with installed_demoapp('install'), chdir('demoapp'):
        out = next(setup_py('--version'))
        exp = '0.0.post0.dev2'
        check_version(out, exp, dirty=False)


def test_git_repo_dirty(tmpdir):  # noqa
    create_demoapp()
    add_tag('demoapp', 'v0.1', 'first tag')
    make_dirty_tree()
    make_commit()
    make_dirty_tree()
    with installed_demoapp('install'), chdir('demoapp'):
        out = next(setup_py('--version'))
        exp = '0.1.post0.dev1'
        check_version(out, exp, dirty=True)


def test_git_repo_with_1_0_tag(tmpdir):  # noqa
    create_demoapp()
    add_tag('demoapp', 'v1.0', 'final release')
    with installed_demoapp('install'), chdir('demoapp'):
        out = next(setup_py('--version'))
        exp = '1.0'
        check_version(out, exp, dirty=False)


def test_git_repo_with_1_0_tag_dirty(tmpdir):  # noqa
    create_demoapp()
    add_tag('demoapp', 'v1.0', 'final release')
    make_dirty_tree()
    with installed_demoapp('install'), chdir('demoapp'):
        out = next(setup_py('--version'))
        exp = '1.0'
        check_version(out, exp, dirty=True)


def test_sdist_install_with_data(tmpdir):  # noqa
    create_demoapp(data=True)
    build_demoapp('sdist', demoapp='demoapp_data')
    with installed_demoapp(demoapp='demoapp_data'):
        out = next(demoapp_data())
        exp = "Hello World"
        assert out.startswith(exp)


def test_bdist_install_with_data(tmpdir):  # noqa
    create_demoapp(data=True)
    build_demoapp('bdist', demoapp='demoapp_data')
    with installed_demoapp('bdist', demoapp='demoapp_data'):
        out = next(demoapp_data())
        exp = "Hello World"
        assert out.startswith(exp)


@pytest.mark.skipif(not is_inside_venv(),  # noqa
                    reason='Needs to run in a virtualenv')
def test_bdist_wheel_install_with_data(tmpdir):
    create_demoapp(data=True)
    build_demoapp('bdist_wheel', demoapp='demoapp_data')
    with installed_demoapp(demoapp='demoapp_data'):
        out = next(demoapp_data())
        exp = "Hello World"
        assert out.startswith(exp)


def test_setup_py_install(tmpdir):  # noqa
    create_demoapp()
    with installed_demoapp('install', demoapp='demoapp'):
        out = next(demoapp('--version'))
        exp = "0.0.post0.dev2"
        check_version(out, exp, dirty=False)



# -*- coding: utf-8 -*-
"""
Command-Line-Interface of PyScaffold
"""
from __future__ import absolute_import, print_function

import argparse
import os.path
import sys
from datetime import date

import pyscaffold

from . import info, repo, shell, structure, templates, utils

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def parse_args(args):
    """
    Parse command line parameters

    :param args: command line parameters as list of strings
    :return: command line parameters as dictionary
    """
    parser = argparse.ArgumentParser(
        description="PyScaffold is a tool for easily putting up the scaffold "
                    "of a Python project.")
    parser.add_argument(
        dest="project",
        help="project name",
        metavar="PROJECT")
    parser.add_argument(
        "-p",
        "--package",
        dest="package",
        required=False,
        help="package name (default: project name)",
        metavar="NAME")
    parser.add_argument(
        "-d",
        "--description",
        dest="description",
        required=False,
        help="package description (default: '')",
        metavar="TEXT")
    parser.add_argument(
        "-u",
        "--url",
        dest="url",
        required=False,
        help="package url (default: '')",
        metavar="URL")
    license_choices = templates.licenses.keys()
    parser.add_argument(
        "-l",
        "--license",
        dest="license",
        choices=license_choices,
        required=False,
        default="none",
        help="package license from {choices} (default: {default})".format(
            choices=str(license_choices), default="No license"),
        metavar="LICENSE")
    parser.add_argument(
        "-f",
        "--force",
        dest="force",
        action="store_true",
        default=False,
        help="force overwriting an existing directory")
    parser.add_argument(
        "-U",
        "--update",
        dest="update",
        action="store_true",
        default=False,
        help="update an existing project by replacing the most important files"
             " like setup.py etc. Use additionally --force to "
             "replace all scaffold files.")
    parser.add_argument(
        "--with-namespace",
        dest="namespace",
        default="",
        help="put your project inside a namespace package",
        metavar="NS1[.NS2]")
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "--with-cookiecutter",
        dest="cookiecutter_template",
        default="",
        metavar="TEMPLATE",
        help="additionally apply a cookiecutter template")
    group.add_argument(
        "--with-django",
        dest="django",
        action="store_true",
        default=False,
        help="generate Django project files")
    parser.add_argument(
        "--with-travis",
        dest="travis",
        action="store_true",
        default=False,
        help="generate Travis configuration files")
    parser.add_argument(
        "--with-pre-commit",
        dest="pre_commit",
        action="store_true",
        default=False,
        help="generate pre-commit configuration file")
    parser.add_argument(
        "--with-tox",
        dest="tox",
        action="store_true",
        default=False,
        help="generate Tox configuration file")
    parser.add_argument(
        "--with-numpydoc",
        dest="numpydoc",
        action="store_true",
        default=False,
        help="add numpydoc to Sphinx configuration file")

    version = pyscaffold.__version__
    parser.add_argument('-v',
                        '--version',
                        action='version',
                        version='PyScaffold {ver}'.format(ver=version))
    opts = vars(parser.parse_args(args))
    return {k: v for k, v in opts.items() if v is not None}


def get_default_opts(project_name, **aux_opts):
    """
    Creates default options using auxiliary options as keyword argument

    Use this function if you want to use PyScaffold from another application
    in order to generate an option dictionary that can than be passed to
    :obj:`create_project`.

    :param project_name: name of the project
    :param aux_opts: auxiliary options as keyword parameters
    :return: options with default values set as dictionary
    """
    # Merge the default options generated by argparse
    opts = parse_args([project_name])
    # Strip (back)slash when added accidentally during update
    opts['project'] = opts['project'].rstrip(os.sep)
    opts.update(aux_opts)
    opts.setdefault('package', utils.make_valid_identifier(opts['project']))
    opts.setdefault('author', info.username())
    opts.setdefault('email', info.email())
    opts.setdefault('release_date', date.today().strftime('%Y-%m-%d'))
    opts.setdefault('year', date.today().year)
    opts.setdefault('license', 'none')
    opts.setdefault('description', 'Add a short description here!')
    opts.setdefault('url', 'http://...')
    opts.setdefault('version', pyscaffold.__version__)
    opts.setdefault('title',
                    '='*len(opts['project']) + '\n' + opts['project'] + '\n' +
                    '='*len(opts['project']))
    classifiers = ['Development Status :: 4 - Beta',
                   'Programming Language :: Python']
    opts.setdefault('classifiers', utils.list2str(
        classifiers, indent=14, brackets=False, quotes=False))
    opts.setdefault('url', 'http://...')
    # Initialize empty list of all requirements
    opts.setdefault('requirements', list())
    opts['namespace'] = utils.prepare_namespace(opts['namespace'])
    if opts['namespace']:
        opts['root_pkg'] = opts['namespace'][0]
    else:
        opts['root_pkg'] = opts['package']
    if opts['update']:
        project_name = opts['project']
        if not os.path.exists(project_name):
            raise RuntimeError(
                "Project {project} does not exist and thus cannot be "
                "updated!".format(project=project_name))
        opts = info.project(opts)
        # Reset project name since the one from setup.cfg might be different
        opts['project'] = project_name
    if opts['django']:
        opts['force'] = True
        opts['package'] = opts['project']  # since this is required by Django
        opts['requirements'].append('django')
    if opts['cookiecutter_template']:
        opts['force'] = True
    return opts


def create_project(opts):
    """
    Create the project's directory structure

    :param opts: options as dictionary
    """
    if opts['django']:
        structure.create_django_proj(opts)
    if opts['cookiecutter_template']:
        structure.create_cookiecutter(opts)
    proj_struct = structure.make_structure(opts)
    structure.create_structure(proj_struct,
                               update=opts['update'] or opts['force'])
    if not opts['update'] and not repo.is_git_repo(opts['project']):
        repo.init_commit_repo(opts['project'], proj_struct)


def make_sanity_checks(opts):
    """
    Perform some sanity checks, e.g., if git is installed.

    :param opts: options as dictionary
    """
    if not info.is_git_installed():
        raise RuntimeError("Make sure git is installed and working.")
    if not info.is_git_configured():
        raise RuntimeError(
            'Make sure git is configured. Run:\n' +
            '  git config --global user.email "you@example.com"\n' +
            '  git config --global user.name "Your Name"\n' +
            "to set your account's default identity.")
    if os.path.exists(opts['project']):
        if not opts['update'] and not opts['force']:
            raise RuntimeError(
                "Directory {dir} already exists! Use --update to update an "
                "existing project or --force to overwrite an existing "
                "directory.".format(dir=opts['project']))


def main(args):
    """
    PyScaffold is a tool for putting up the scaffold of a Python project.
    """
    opts = parse_args(args)
    opts = get_default_opts(opts['project'], **opts)
    make_sanity_checks(opts)
    create_project(opts)
    if opts['update'] and not opts['force']:
        note = "Update accomplished!\n" \
               "Please check if your setup.cfg still complies with:\n" \
               "http://pyscaffold.readthedocs.org/en/v{}/configuration.html"
        print(note.format(pyscaffold.__version__))


@shell.called_process_error2exit_decorator
@utils.exceptions2exit([RuntimeError])
def run():
    """
    Entry point for setup.py
    """
    main(sys.argv[1:])


if __name__ == '__main__':
    main(sys.argv[1:])



# -*- coding: utf-8 -*-
"""
Provide general information about the system, user etc.
"""
from __future__ import absolute_import, print_function

import copy
import getpass
import os
import socket
from subprocess import CalledProcessError

from six.moves import configparser

from . import shell, utils

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def username():
    """
    Retrieve the user's name

    :return: user's name as string
    """
    try:
        user = next(shell.git("config", "--global", "--get", "user.name"))
        user = user.strip()
    except CalledProcessError:
        user = getpass.getuser()
    return utils.utf8_decode(user)


def email():
    """
    Retrieve the user's email

    :return: user's email as string
    """
    try:
        email = next(shell.git("config", "--global", "--get", "user.email"))
        email = email.strip()
    except CalledProcessError:
        user = getpass.getuser()
        host = socket.gethostname()
        email = "{user}@{host}".format(user=user, host=host)
    return utils.utf8_decode(email)


def is_git_installed():
    """
    Check if git is installed

    :return: boolean
    """
    try:
        shell.git("--version")
    except CalledProcessError:
        return False
    return True


def is_git_configured():
    """
    Check if user.name and user.email is set globally in git

    :return: boolean
    """
    try:
        for attr in ["name", "email"]:
            shell.git("config", "--global", "--get", "user.{}".format(attr))
    except CalledProcessError:
        return False
    return True


def project(opts):
    """
    Update user options with the options of an existing PyScaffold project

    :param opts: options as dictionary
    :return: options with updated values as dictionary
    """
    opts = copy.copy(opts)
    try:
        config = configparser.SafeConfigParser()
        config.read(os.path.join(opts['project'], 'setup.cfg'))
        # Some branches due to backward compatibility
        if config.has_option('metadata', 'name'):
            opts['project'] = config.get('metadata', 'name')
        if config.has_option('metadata', 'description'):
            opts['description'] = config.get('metadata', 'description')
        else:
            opts['description'] = config.get('metadata', 'summary')
        opts['author'] = config.get('metadata', 'author')
        if config.has_option('metadata', 'author_email'):
            opts['email'] = config.get('metadata', 'author_email')
        else:
            opts['email'] = config.get('metadata', 'author-email')
        opts['license'] = utils.best_fit_license(
            config.get('metadata', 'license'))
        if config.has_option('metadata', 'url'):
            opts['url'] = config.get('metadata', 'url')
        else:
            opts['url'] = config.get('metadata', 'home-page')
        if config.has_option('metadata', 'classifiers'):
            opts['classifiers'] = config.get('metadata', 'classifiers')
        if config.has_option('files', 'packages'):
            opts['package'] = config.get('files', 'packages').strip()
    except Exception as e:
        print(e)
        raise RuntimeError("Could not update {project}. Was it generated "
                           "with PyScaffold?".format(project=opts['project']))
    return opts



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import imp
import os
import sys

import pytest
from pyscaffold import cli

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def test_parse_args():
    args = ["my-project"]
    opts = cli.parse_args(args)
    assert opts['project'] == "my-project"


def test_main_with_nogit(nogit_mock):  # noqa
    args = ["my-project"]
    with pytest.raises(RuntimeError):
        cli.main(args)


def test_main_with_git_not_configured(noconfgit_mock):  # noqa
    args = ["my-project"]
    with pytest.raises(RuntimeError):
        cli.main(args)


def test_main_when_folder_exists(tmpdir, git_mock):  # noqa
    args = ["my-project"]
    os.mkdir(args[0])
    with pytest.raises(RuntimeError):
        cli.main(args)


def test_main(tmpdir, git_mock):  # noqa
    args = ["my-project"]
    cli.main(args)
    assert os.path.exists(args[0])


def test_main_when_updating(tmpdir, git_mock):  # noqa
    args = ["my-project"]
    cli.main(args)
    args = ["--update", "my-project"]
    cli.main(args)
    assert os.path.exists(args[1])


def test_main_when_updating_with_wrong_setup(tmpdir, git_mock):  # noqa
    os.mkdir("my_project")
    open("my_project/setup.py", 'a').close()
    args = ["--update", "my_project"]
    with pytest.raises(RuntimeError):
        cli.main(args)


def test_main_when_updating_project_doesnt_exist(tmpdir, git_mock):  # noqa
    args = ["--update", "my_project"]
    with pytest.raises(RuntimeError):
        cli.main(args)


def test_main_with_license(tmpdir, git_mock):  # noqa
    args = ["my-project", "-l", "new-bsd"]
    cli.main(args)
    assert os.path.exists(args[0])


def test_run(tmpdir, git_mock):  # noqa
    sys.argv = ["pyscaffold", "my-project"]
    cli.run()
    assert os.path.exists(sys.argv[1])


def test_overwrite_git_repo(tmpdir):  # noqa
    sys.argv = ["pyscaffold", "my_project"]
    cli.run()
    with pytest.raises(SystemExit):
        cli.run()
    sys.argv = ["pyscaffold", "--force", "my_project"]
    cli.run()


def test_overwrite_dir(tmpdir):  # noqa
    os.mkdir("my_project")
    sys.argv = ["pyscaffold", "--force", "my_project"]
    cli.run()


def test_django_proj(tmpdir):  # noqa
    sys.argv = ["pyscaffold", "--with-django", "my_project"]
    cli.run()


def test_with_numpydoc(tmpdir):  # noqa
    sys.argv = ["pyscaffold", "--with-numpydoc", "my_project"]
    cli.run()
    conffile = os.path.join(
        os.path.abspath(os.path.curdir), "my_project", "docs", "conf.py")
    conf = imp.load_source('conf', conffile)
    assert sorted(conf.extensions) == sorted(['sphinx.ext.autodoc',
                                              'sphinx.ext.intersphinx',
                                              'sphinx.ext.todo',
                                              'sphinx.ext.autosummary',
                                              'sphinx.ext.viewcode',
                                              'sphinx.ext.coverage',
                                              'sphinx.ext.doctest',
                                              'sphinx.ext.ifconfig',
                                              'sphinx.ext.pngmath',
                                              'numpydoc'])


def test_with_travis(tmpdir):  # noqa
    sys.argv = ["pyscaffold", "--with-travis", "my_project"]
    cli.run()


def test_with_namespaces(tmpdir):  # noqa
    sys.argv = ["pyscaffold", "--with-namespace", "com.blue_yonder",
                "my_project"]
    cli.run()
    assert os.path.exists("my_project/com/blue_yonder")


def test_get_default_opts():
    args = ["project", "-p", "package", "-d", "description"]
    opts = cli.parse_args(args)
    new_opts = cli.get_default_opts(opts['project'], **opts)
    assert "author" not in opts
    assert "author" in new_opts


def test_get_defaults_opts_with_cookiecutter():
    args = ["project", "--with-cookiecutter", "http://..."]
    opts = cli.parse_args(args)
    new_opts = cli.get_default_opts(opts['project'], **opts)
    assert new_opts["force"]


def test_api(tmpdir):  # noqa
    opts = cli.get_default_opts('created_proj_with_api')
    cli.create_project(opts)
    assert os.path.exists('created_proj_with_api')


def test_api_with_cookiecutter(tmpdir):  # noqa
    template = 'https://github.com/audreyr/cookiecutter-pypackage.git'
    opts = cli.get_default_opts('created_proj_with_api',
                                cookiecutter_template=template)
    cli.create_project(opts)
    assert os.path.exists('created_proj_with_api')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function

import getpass
import os
import socket

import pytest
from pyscaffold import info, cli
from six import string_types

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def test_username_with_git(git_mock):  # noqa
    username = info.username()
    assert isinstance(username, string_types)
    assert len(username) > 0


def test_username_with_no_git(nogit_mock):  # noqa
    username = info.username()
    assert isinstance(username, string_types)
    assert getpass.getuser() == username


def test_email_with_git(git_mock):  # noqa
    email = info.email()
    assert "@" in email


def test_email_with_nogit(nogit_mock):  # noqa
    email = info.email()
    assert socket.gethostname() == email.split("@")[1]


def test_git_is_installed(git_mock):  # noqa
    assert info.is_git_installed()


def test_git_is_not_installed(nogit_mock):  # noqa
    assert not info.is_git_installed()


def test_is_git_configured(git_mock):  # noqa
    assert info.is_git_configured()


def test_is_git_not_configured(noconfgit_mock):  # noqa
    assert not info.is_git_configured()


def test_project_raises():
    opts = {"project": "non_existant"}
    with pytest.raises(RuntimeError):
        info.project(opts)


def test_project_without_args(tmpdir):  # noqa
    old_args = ["my_project", "-u", "http://www.blue-yonder.com/",
                "-d", "my description"]
    cli.main(old_args)
    args = ["my_project"]
    opts = cli.parse_args(args)
    new_opts = info.project(opts)
    assert new_opts['url'] == "http://www.blue-yonder.com/"
    assert new_opts['package'] == "my_project"
    assert new_opts['license'] == "none"
    assert new_opts['description'] == "my description"


def test_project_with_args(tmpdir):  # noqa
    old_args = ["my_project", "-u", "http://www.blue-yonder.com/",
                "-d", "my description"]
    cli.main(old_args)
    args = ["my_project", "-u", "http://www.google.com/",
            "-d", "other description"]
    opts = cli.parse_args(args)
    new_opts = info.project(opts)
    assert new_opts['url'] == "http://www.blue-yonder.com/"
    assert new_opts['package'] == "my_project"
    assert new_opts['description'] == "my description"


def test_project_with_no_setup(tmpdir):  # noqa
    os.mkdir("my_project")
    args = ["my_project"]
    args = cli.parse_args(args)
    with pytest.raises(RuntimeError):
        info.project(args)


def test_project_with_wrong_setup(tmpdir):  # noqa
    os.mkdir("my_project")
    open("my_project/setup.py", 'a').close()
    args = ["my_project"]
    args = cli.parse_args(args)
    with pytest.raises(RuntimeError):
        info.project(args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    Setup file for PyScaffold.

    Important note: Since PyScaffold is self-using and depends on
    setuptools-scm, it is important to run `python setup.py egg_info` after
    a fresh checkout. This will generate some critically needed data.
"""

import inspect
import os
import sys
from distutils.cmd import Command

from setuptools import setup, find_packages

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"
__location__ = os.path.join(os.getcwd(), os.path.dirname(
    inspect.getfile(inspect.currentframe())))


def build_cmd_docs():
    try:
        from sphinx.setup_command import BuildDoc
    except ImportError:
        class NoSphinx(Command):
            user_options = []

            def initialize_options(self):
                raise RuntimeError("Sphinx documentation is not installed, "
                                   "run: pip install sphinx")

        return NoSphinx
    else:
        return BuildDoc


def read(fname):
    with open(os.path.join(__location__, fname)) as fh:
        content = fh.read()
    return content


def version2str(version):
    if version.exact or not version.distance > 0:
        return version.format_with('{tag}')
    else:
        distance = version.distance
        version = str(version.tag)
        if '.dev' in version:
            version, tail = version.rsplit('.dev', 1)
            assert tail == '0', 'own dev numbers are unsupported'
        return '{}.post0.dev{}'.format(version, distance)


def local_version2str(version):
    if version.exact:
        if version.dirty:
            return version.format_with('+dirty')
        else:
            return ''
    else:
        if version.dirty:
            return version.format_with('+{node}.dirty')
        else:
            return version.format_with('+{node}')


def get_install_requirements(path):
    with open(os.path.join(__location__, path)) as fh:
        content = fh.read()
    return [req for req in content.splitlines() if req != '']


def setup_package():
    docs_path = os.path.join(__location__, 'docs')
    docs_build_path = os.path.join(docs_path, '_build')
    needs_pytest = {'pytest', 'test', 'ptr'}.intersection(sys.argv)
    pytest_runner = ['pytest-runner'] if needs_pytest else []
    requirements = get_install_requirements('requirements.txt')
    test_command = {'addopts': ('setup.py', 'tests '
                                            '--cov pyscaffold '
                                            '--cov-report term-missing '
                                            '--verbose')}
    command_options = {
        'docs': {'build_dir': ('setup.py', docs_build_path),
                 'config_dir': ('setup.py', docs_path),
                 'source_dir': ('setup.py', docs_path)},
        'doctest': {'build_dir': ('setup.py', docs_build_path),
                    'config_dir': ('setup.py', docs_path),
                    'source_dir': ('setup.py', docs_path),
                    'builder': ('setup.py', 'doctest')},
        'test': test_command,
        'pytest': test_command
    }
    entry_points = {
        'console_scripts': ['putup=pyscaffold.cli:run'],
        'distutils.setup_keywords':
            ['use_pyscaffold=pyscaffold.integration:pyscaffold_keyword']
    }
    setup(name='pyscaffold',
          url='http://pyscaffold.readthedocs.org/',
          author='Florian Wilhelm',
          author_email='florian.wilhelm@blue-yonder.com',
          description='Tool for easily putting up the scaffold of a '
                      'Python project',
          long_description=read('README.rst'),
          license='new BSD',
          classifiers=['Development Status :: 5 - Production/Stable',
                       'Topic :: Utilities',
                       'Programming Language :: Python',
                       'Programming Language :: Python :: 2',
                       'Programming Language :: Python :: 2.7',
                       'Programming Language :: Python :: 3',
                       'Programming Language :: Python :: 3.3',
                       'Programming Language :: Python :: 3.4',
                       'Environment :: Console',
                       'Intended Audience :: Developers',
                       'License :: OSI Approved :: BSD License',
                       'Operating System :: POSIX :: Linux',
                       'Operating System :: Unix',
                       'Operating System :: MacOS',
                       'Operating System :: Microsoft :: Windows'],
          setup_requires=requirements + pytest_runner,
          tests_require=['pytest-cov', 'pytest'],
          install_requires=requirements,
          package_data={'pyscaffold': ['data/*']},
          cmdclass={'docs': build_cmd_docs(), 'doctest': build_cmd_docs()},
          command_options=command_options,
          entry_points=entry_points,
          packages=find_packages(exclude=['tests', 'tests.*']),
          use_scm_version={'version_scheme': version2str,
                           'local_scheme': local_version2str})


if __name__ == '__main__':
    setup_package()



# -*- coding: utf-8 -*-

"""
Integration part for hooking into distutils/setuptools

Rationale:
The ``use_pyscaffold`` keyword is unknown to setuptools' setup(...) command,
therefore the ``entry_points`` are checked for a function to handle this
keyword which is ``pyscaffold_keyword`` below. This is where we hook into
setuptools and apply the magic of setuptools_scm and pbr.
"""

from __future__ import division, print_function, absolute_import

import os
from distutils.cmd import Command

from setuptools_scm import get_version
from setuptools_scm.utils import trace
from setuptools_scm.version import _warn_if_setuptools_outdated
from pbr.core import pbr as read_setup_cfg

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


def version2str(version):
    """
    Creates a PEP440 version string

    :param version: version object as :obj:`setuptools_scm.version.ScmVersion`
    :return: version string
    """
    if version.exact or not version.distance > 0:
        return version.format_with('{tag}')
    else:
        distance = version.distance
        version = str(version.tag)
        if '.dev' in version:
            version, tail = version.rsplit('.dev', 1)
            assert tail == '0', 'own dev numbers are unsupported'
        return '{}.post0.dev{}'.format(version, distance)


def local_version2str(version):
    """
    Create the local part of a PEP440 version string

    :param version: version object as :obj:`setuptools_scm.version.ScmVersion`
    :return: local version string
    """
    if version.exact:
        return ''
    else:
        if version.dirty:
            return version.format_with('+{node}.dirty')
        else:
            return version.format_with('+{node}')


def deactivate_pbr_authors_changelog():
    """
    Deactivate automatic generation of AUTHORS and ChangeLog file

    This is an automatism of PBR and we rather keep track of our own
    AUTHORS.rst and CHANGES.rst files.
    """
    os.environ['SKIP_GENERATE_AUTHORS'] = "1"
    os.environ['SKIP_WRITE_GIT_CHANGELOG'] = "1"


def build_cmd_docs():
    """
    Return Sphinx's BuildDoc if available otherwise a dummy command

    :return: command as :obj:`~distutils.cmd.Command`
    """
    try:
        from sphinx.setup_command import BuildDoc
    except ImportError:
        class NoSphinx(Command):
            user_options = []

            def initialize_options(self):
                raise RuntimeError("Sphinx documentation is not installed, "
                                   "run: pip install sphinx")

        return NoSphinx
    else:
        return BuildDoc


def pyscaffold_keyword(dist, keyword, value):
    """
    Handles the `use_pyscaffold` keyword of the setup(...) command

    :param dist: distribution object as :obj:`setuptools.dist`
    :param keyword: keyword argument = 'use_pyscaffold'
    :param value: value of the keyword argument
    """
    _warn_if_setuptools_outdated()
    if value is True:
        deactivate_pbr_authors_changelog()
        read_setup_cfg(dist, keyword, value)
        try:
            dist.metadata.version = get_version(version_scheme=version2str,
                                                local_scheme=local_version2str)
        except Exception as e:
            trace('error', e)
        # Adding doctest again since PBR seems to drop these
        dist.cmdclass['doctest'] = build_cmd_docs()
        dist.command_options['doctest'] = {'builder': ('setup.py', 'doctest')}



# -*- coding: utf-8 -*-
"""
Shell commands like git, django-admin.py etc.
"""

from __future__ import absolute_import, division, print_function

import functools
import subprocess
import sys

__author__ = "Florian Wilhelm"
__copyright__ = "Blue Yonder"
__license__ = "new BSD"


class ShellCommand(object):
    """
    Shell command that can be called with flags like git('add', 'file')

    :param command: command to handle
    :param shell: run the command in the shell
    :param cwd: current working dir to run the command
    """
    def __init__(self, command, shell=True, cwd=None):
        self._command = command
        self._shell = shell
        self._cwd = cwd

    def __call__(self, *args):
        command = "{cmd} {args}".format(cmd=self._command,
                                        args=subprocess.list2cmdline(args))
        output = subprocess.check_output(command,
                                         shell=self._shell,
                                         cwd=self._cwd,
                                         stderr=subprocess.STDOUT,
                                         universal_newlines=True)
        return self._yield_output(output)

    def _yield_output(self, msg):
        for line in msg.splitlines():
            yield line


def called_process_error2exit_decorator(func):
    """
    Decorator to convert given CalledProcessError to an exit message

    This avoids displaying nasty stack traces to end-users
    """
    @functools.wraps(func)
    def func_wrapper(*args, **kwargs):
        try:
            func(*args, **kwargs)
        except subprocess.CalledProcessError as e:
            print("{err}:\n{msg}".format(err=str(e), msg=e.output))
            sys.exit(1)
    return func_wrapper


def get_git_cmd(**args):
    """
    Retrieve the git shell command depending on the current platform

    All additional parameters are passed to :obj:`~.ShellCommand`
    """
    if sys.platform == "win32":
        for cmd in ["git.cmd", "git.exe"]:
            git = ShellCommand(cmd, **args)
            try:
                git("--version")
            except subprocess.CalledProcessError:
                continue
            return git
        return None
    else:
        git = ShellCommand("git", **args)
        try:
            git("--version")
        except subprocess.CalledProcessError:
            return None
        return git


#: Command for git
git = get_git_cmd()

#: Command for django-admin.py
django_admin = ShellCommand("django-admin.py")
